/****************************************************************************
 *   Copyright (C) 2004 by Jurij Ivastsuk-Kienbaum                          *
 *   jurij@device-image.de                                                  *
 *                                                                          *
 *   This program is free software; you can redistribute it and/or modify   *
 *   it under the terms of the GNU General Public License as published by   *
 *   the Free Software Foundation; either version 2 of the License,         *
 *   or (at your option) any later version.                                 *
 *   This program is distributed in the hope that it will be useful,        *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty            *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                *
 *   See the GNU General Public License for more details.                   *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to the Free Software Foundation,*
 *   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA            *
 ***************************************************************************/
#include "unzsplit.h"

using namespace std;

int file_count=0;  	// amount of input files
int ifd = 0;		// input-file descritor
int ofd = 0;		// output-file descritor
int opath = 0;		// out-path exists or not
int ipath = 0;		// in-path exists or not
int out_filename = 0;	// output-filename as parameter exists or not
int out_device = 0;	// indicator whether output device as parameter exists or not
int chunk_size_known = 0; // tells if the chunk-size as parameter delivered or not

int debug = 0;		// if debug == 1, debug information will be written to debug.log file
int noerror = 0;	// switch to signal if the read operation should proceed after I/O error (0 = no; 1 = yes)
int multivol = 0;	// switch which activates the multivolume feature

unsigned int safe_seek_block = 512;

unsigned int buffer_size = 64;

unsigned int io_block_size = 65536;	// the size of read/write buffer in bytes

off_t ifile_size;      		//input file size, -1 for devices (debug only)
off_t chunk_size;  		// size of the splited part in bytes

float init_value;		// size of splited part (parameter as float number)
string progname;   		// program name
char ifname[MAX_PATH_LEN]; 	// input file name
int to_stdout = 0;    		// output to stdout (-c)
int from_stdin = 0;		// input from stdin

unsigned int first_chunk = 0;

string out_filen;	// output filename
string out_devn;	// output device

vector<string> names_cont;

string out_pathn;	// output path name
string in_pathn;	// input path name

string OutputFilename;	// the main Name of Output File,


char log_buf[1024];
string log_str;

struct stat istat;


int level = 6;	// zip level (from 1 to 9) 1 - fast 9 - best
		// default value is 6

unsigned long long end_sum_read = 0;
unsigned long long end_sum_written = 0;


unsigned long end_sum_elps_time = 0;


string total_time_elps;


string av_speed_s;

string version = "1.2.0";
string vers_info = "Copyright (C) 2005 Jurij Ivastsuk-Kienbaum.";
string copyleft = "Written by Jurij Ivastsuk-Kienbaum.\nCopyright (C) 2005 Jurij Ivastsuk-Kienbaum.\nThis program comes with ABSOLUTELY NO WARRANTY.\nThis is free software, and you are welcome to redistribute it under conditions of GNU GPL.";


struct option longopts[] =
{
  /*{name  has_arg *flag val}*/
  {"help",	0, 0, 'h'}, // help, extended usage information
  {"usage",	0, 0, '?'}, // help, usage information
  {"buffer",	1, 0, 'b'}, // read/write buffer size (only for experiments!)
  {"debug",	0, 0, 'd'}, // debug, some information about read and written bytes will be spooled to debug.log file
  {"stdout",	0, 0, 'c'}, // write output to standard output
  {"outdir",	1, 0, 'o'}, // output directory
  {"indir",	1, 0, 'i'}, // input directory
  {"multivol",	0, 0, 'm'}, // multivolume feature, which allows replacing removable volumes and continue restore operation
  {"outname",	1, 0, 'N'}, // file name which will be used as output file name
  {"outdev",	1, 0, 'D'}, // device which will be used as output target
  {"noerror",	0, 0, 'r'}, 	// Ignore i/o errors and continue after read errors; pad every input block with NULs
  				// Zero the buffer before reading, so that if we get a read error,
	 			// whatever data we are able to read is followed by zeros.
	 			// This minimizes data loss.
  {"version",	1, 0, 'v'}, // program version
  {0, 0, 0, 0}
};

int main(int argc, char ** argv)
 {


 	int rc = 0;
	int res =0;
        int optc;

 	progname = cut_path(argv[0]);

	OutputFilename = get_def_filename();

	/* get options */
 	while (( optc = getopt_long(argc,argv,"h?dcmb:o:i:N:D:vr", longopts, (int *)0)) != -1 )
  	{
  		switch (optc)
   		{
  		 case 'N' :
        		out_filen.assign(optarg);
			out_filename = 1;
                        break;
		 case 'b':
		 	sscanf(optarg, "%d", &buffer_size);
			io_block_size = 512 * 2 * buffer_size;
			break;
		 case 'D' :
        		out_devn.assign(optarg);
			out_device = 1;
                        break;
                 case 'o' :
			out_pathn.assign(optarg);


			out_pathn = add_last_slash(out_pathn);
	    		opath = 1;
                        break;
		 case 'i' :
			in_pathn.assign(optarg);


			in_pathn = add_last_slash(in_pathn);
	    		ipath = 1;
                        break;
		 case 'c' :
			to_stdout = 1;
                        break;
		 case 'd' :
			debug = 1;
                        break;
		 case 'r' :
			noerror = 1;
                        break;
		 case 'm' :
		 	multivol = 1;
			break;
		 case 'v' :
    		 	fprintf(stdout, "%s version: %s\n%s\n", progname.c_str(), version.c_str(), vers_info.c_str());
    			rc = -1;
    			do_exit(rc);
   		 case 'h' :
		 	help();
    			rc = -1;
    			do_exit(rc);
   		 case '?' :
    		 	usage();
    			rc = -1;
    			do_exit(rc);
                 default:
	    		/* Error */
	    		usage();
			rc = -1;
	    		do_exit(rc);
   		}
  	}


	if(out_device == 1)
	{
		opath = 0;
		out_filename = 0;
		to_stdout = 0;
	}
	else
	{
		if(out_filename == 1 || opath == 1)
		{
			to_stdout = 0;
		}
	}

        file_count = argc - optind;

 	if ( file_count == 1 )
  	{
		if(debug == 1)
		{
			sprintf(log_buf, "%s: version: %s", progname.c_str(), version.c_str());
			log_str = log_buf;
			log(log_str);
			sprintf(log_buf, "%s: start of operation", progname.c_str());
			log_str = log_buf;
			log(log_str);
			sprintf(log_buf, "%s: read/write buffer size: %d kibibyte (%d bytes).", progname.c_str(), buffer_size, io_block_size);
			log_str = log_buf;
			log(log_str);

			if (noerror == 1)
			{
				sprintf(log_buf, "%s: \"--noerror\" option in use, I/O read errors will be savely ignored", progname.c_str());
				log_str = log_buf;
				log(log_str);
			}
		}
		else
		{
			fprintf(stderr,"%s: version: %s\n", progname.c_str(), version.c_str());
			fprintf(stderr,"%s: start of operation\n", progname.c_str());
			fprintf(stderr,"%s: read/write buffer size: %d kibibyte (%d bytes).\n", progname.c_str(), buffer_size, io_block_size);
			if (noerror == 1)
			{
				fprintf(stderr,"%s: \"--noerror\" option in use, I/O read errors will be savely ignored\n", progname.c_str());
			}
		}

                res = treat_file(argv[optind]);

		if(res == 0)
		{
                        if(debug == 1)
  			{
                                sprintf(log_buf, "%s: Success!...", progname.c_str());
				log_str = log_buf;
				log(log_str);
				sprintf(log_buf, "%s: end of operation\n", progname.c_str());
				log_str = log_buf;
				log(log_str);
  			}
			else
			{
                        	fprintf(stderr,"%s: Success!...\n", progname.c_str());
				fprintf(stderr,"%s: end of operation\n", progname.c_str());
			}
			rc = 0;
	    		do_exit(rc);
		}
		else
		{
			if(debug == 1)
  			{
                                sprintf(log_buf, "%s: not succeeded..., error in operation!", progname.c_str());
				log_str = log_buf;
				log(log_str);
				sprintf(log_buf, "%s: end of operation\n", progname.c_str());
				log_str = log_buf;
				log(log_str);
  			}
			else
			{
				fprintf(stderr,"%s: not succeeded..., error in operation!\n", progname.c_str());
				fprintf(stderr,"%s: end of operation\n", progname.c_str());
			}
			rc = -1;
	    		do_exit(rc);
		}

	}
        else // error
        {
		usage();
		rc = -1;
	    	do_exit(rc);
        }

 } // END main


void usage()
 {
        if(debug == 1)
	{
        	sprintf(log_buf, "%s: usage: [-h?dm] [-i input_dir] [-c stdout] [-o output_dir] [-D output_dev] [-N outname] [file [- stdin]]", progname.c_str());
		log_str = log_buf;
		log(log_str);
	}
	else
	{
		fprintf (stderr, "%s: usage: [-h?dm] [-i input_dir] [-c stdout] [-o output_dir] [-D output_dev] [-N outname] [file [- stdin]]\n", progname.c_str());
	}
 }

  void help()
 {
 	static const char help_str[] =
	"unzsplit - will read splitted files produced by zsplit, decompress and glue them together to its origin.\n"
	"\n"
	"usage: unzsplit [OPTIONS] SOURCE\n"
	"\n"
	"Reads from SOURCE, which can be one, or a list of splitted and compressed files produced\n"
	"by zsplit, make decompress and glue them in a right order together. The result of operation is the\n"
	"uncompressed original big file or restored device.\n"
	"To read the list of files it is sufficient to use a common part of the file names as a SOURCE.\n"
	"Unzsplit recognizes all files which belongs to the list and the right order of reading\n"
	"of file-chunks if file names remain the same as created by zsplit (see example).\n"
	"It is absolutely necessarily that the file name containes the end part:\n"
	"_[digit].spl.zp (such as: _0.spl.zp or _1.spl.zp ...) because this [digit] is used to find\n"
	"a right reading order of file chunks. The [digit] of the first file should always be 0.\n"
	"\n"
	"SOURCE:\n"
	"can be a file (example: some_name_0.spl.zp), a file list\n"
	"(example: some_name_0.spl.zp, some_name_1.spl.zp, ...)  or stdin (-).\n"
	"\"some_name\" is a common part of names of the file list.\n"
        "\n"
	"OPTIONS:\n"
	" -b, --buffer SIZE:             Setup read/write buffer SIZE (in kibibytes) to optimize read/write speed.\n"
	"                                Example of SIZE: 2 (2048 bytes).\n"
	"                                If this option is not used, the default size is 64 (65536 bytes).\n\n"
	" -c, --stdout :			Write output to stdout (standard output).\n\n"
	" -D, --outdev :			Write output to output device.\n\n"
	" -d, --debug :			Debug information will be written into the debug.log file in the current directory.\n"
	"				If this option is not used, the debug information will be printed to stderr.\n\n"
	" -i, --indir DIR :		Input directory (DIR is directory which containes the SOURCE).\n\n"
	" -m, --multivol :		Switch the multivolume feature on.\n\n"
	" -N, --outname OUT_NAME :	The name of the output file.\n\n"
	" -h, --help :			Shows this message.\n\n"
	" -o, --outdir DIR :		Output directory (DIR is directory which containes the output files).\n\n"
	" -r, --noerror:                 If during the read operation I/O error happens, safely ignore these errors and read farther.\n\n"
	"EXAMPLES:\n"
	"unzsplit -D /dev/hda -d WinXP_bkp\n"
	"	unzsplit reads the file list in a current directory with a common name-part WinXP_bkp,\n"
	"	such as: WinXP_bkp_0.spl.zp, WinXP_bkp_1.spl.zp, WinXP_bkp_2.spl.zp ...,\n"
	"	uncompresses and glue them together to one image, and restore this image to the device /dev/hda.\n"
	"	The debug information is written into the debug.log file in the current directory.\n\n"
	"unzsplit -N test_img -o /home/user -d -\n"
	"	unzsplit reads from stdin, decompresses using default decompression method,\n"
	"	creates an uncompressed image file test_img.img into the directory\n"
	"	/home/user and writes the debug information to the debug.log file in the current directory.\n\n"
	"unzsplit -N test_bkp WinXP_bkp\n"
	"	unzsplit reads list of files with a common name-part WinXP_bkp in a current directory,\n"
	"	decompresses using default decompression method, ties all uncompressed parts in a right order\n"
	"	together, saves the output file as test_bkp.img in the current directory and\n"
	"	writes the debug information to the console (stderr).\n"
	"\n";
 	fprintf(stdout, "%s: version %s\n%s\n\n", progname.c_str(), version.c_str(), copyleft.c_str());
	fprintf(stdout,help_str);
 }


string av_speed(unsigned long time_sec, unsigned long long total_bytes_read)
{
	string ret_av_speed;
	float min=0.00f;
	float bytes_min=0.00f;
	float mbytes_min=0.00f;
	ostringstream av_sp_mb_min;
	
	min = time_sec/60.00f;
	
	bytes_min = total_bytes_read/min;
	
	if(time_sec == 0 || total_bytes_read == 0)
	{
		mbytes_min = 0;
	}
	else
	{
		mbytes_min = bytes_min/(1024*1024);
	}
	
	av_sp_mb_min << mbytes_min << "MiB/min";
	
	ret_av_speed = av_sp_mb_min.str();
	
	return ret_av_speed;
}

unsigned int find_last_chunk_nr(vector<string> &cont)
{
	unsigned int last_nbr=0;
	vector<string>::iterator it_n;
   	int pos1, pos2;
   	unsigned int chunk_nbr=0;
   	string temp_name, temp_substr, dig;

	for(it_n = cont.begin(); it_n != cont.end(); it_n++)
	{
		temp_name = *it_n;

		pos1 = temp_name.find_last_of("_");
		if(pos1 != -1)
  		{
        		temp_substr = temp_name.substr(pos1+1);

			pos2 = temp_substr.find(".");
			if(pos2 != -1)
			{

				dig = temp_substr.substr(0, pos2);
				chunk_nbr = atoi(dig.c_str());
			}
			if(chunk_nbr > last_nbr)
			{
                        	last_nbr = chunk_nbr;
			}
		}
	}

        return last_nbr;
}

string check_get_name(string strname)
{
  string temp_name, temp_substr, ret_substr;
  int pos1, pos2;


  temp_name = strname;

  pos1 = temp_name.find_last_of("_");

  if(pos1 != -1)
  {
        temp_substr = temp_name.substr(pos1);
	pos2 = temp_substr.find_first_of("1234567890", 1);
	if(pos2 != -1)
	{
		ret_substr = temp_name.substr(0, pos1);
	}
	else
	{
        	ret_substr = strname;
	}

  }
  else
  {

	ret_substr = strname;
  }

  pos1 = ret_substr.find(".");
  if(pos1 != -1)
  {
  	ret_substr = ret_substr.erase(pos1);
  }

  return ret_substr;
}

string find_chunk_name(vector<string> &cont, string name_patt)
{
  string res;
  int pos;
  vector<string>::iterator it;
  if(cont.size() != 0)
  {
  	for(it = cont.begin(); it != cont.end(); it++)
	{
        	pos = (*it).find(name_patt);
		if(pos != -1)
		{
			res = *it;
			break;
		}
		else
		{
                	res = "";
		}
	}
  }

  return res;
}


int search_file_parts(string iname, unsigned int chunk_count)
{
   int res = 0;
   int find_res; 	// counter for find in string operation
   int found_names = 0;		// indicator for found files
   struct dirent **namelist; 	// structure which containes file-names (for findig files in the directory)
   string part_fname; 		// name of the file chunk
   int i; 	//counter
   vector<string> names_pre_cont; // container for unsorted filenames
   string real_fpname;		// real file part-name without the path-part
   string dir_path;
   unsigned int indx = 0;		// file-name pattern counter
   string name_patt;  		// file-name pattern
   ostringstream i_str; 	// stream object for doing sprintf-like operations
   int con_check = 0;		// consistency checker
   string tmp_chunk_name;
   unsigned int last_chunk_nbr = 0; // the found last chunk number

   vector<string>::iterator it_n; // iterator for out-put names in names_cont, for check consistency
   int pos1, pos2;
   int chunk_nbr = 0;
   string temp_name, temp_substr, dig;

	con_check = chunk_count;


	if(ipath == 1)
	{
		dir_path = in_pathn;
	}
	else
	{
		dir_path = get_path(iname);
		if(dir_path == "")
		{
			dir_path = ".";
		}
	}


	found_names = scandir(dir_path.c_str(), &namelist, 0, alphasort);


	names_pre_cont.clear();
	names_cont.clear();

	real_fpname = cut_path(iname);




	real_fpname = check_get_name(real_fpname);

	OutputFilename = real_fpname;



	if(found_names > 0)
	{
		for(i=0; i <= found_names-1; i++)
		{
			part_fname.assign(namelist[i]->d_name);
			find_res = part_fname.find(real_fpname);
			if(find_res != -1)
			{
                               names_pre_cont.push_back(part_fname);
			}
		}
	}

  if(names_pre_cont.size() != 0)
  {
	last_chunk_nbr = find_last_chunk_nr(names_pre_cont);


	first_chunk = last_chunk_nbr + 1;





	if( last_chunk_nbr < chunk_count)
	{
		last_chunk_nbr = chunk_count;
	}

  	for(indx = chunk_count; indx <= last_chunk_nbr+1; indx++)
	{
		name_patt.assign("_");
		i_str << indx;
		name_patt.append(i_str.str());

		i_str.str("");
		name_patt.append(".spl.zp");
		tmp_chunk_name = find_chunk_name(names_pre_cont, name_patt);

		if(tmp_chunk_name != "")
		{
			names_cont.push_back(tmp_chunk_name);
		}


		if(indx == chunk_count && tmp_chunk_name == "")
		{
			res = -2;
			return res;
		}
	}


	for(it_n = names_cont.begin(); it_n != names_cont.end(); it_n++)
	{
		temp_name = *it_n;

		pos1 = temp_name.find_last_of("_");
		if(pos1 != -1)
  		{
        		temp_substr = temp_name.substr(pos1+1);

			pos2 = temp_substr.find(".");
			if(pos2 != -1)
			{

				dig = temp_substr.substr(0, pos2);
				chunk_nbr = atoi(dig.c_str());
			}
			if(chunk_nbr != con_check)
			{
				res = -3;
				break;
			}
		}
		else
		{
        		res = -3;
			break;
		}
	  con_check++;
	}


    	if(names_cont.size() == 0)
    	{
    		res = -1;
    	}

    names_pre_cont.clear();
  }
  else
  {
    res = -1;
  }

  return res;
}
void close_in_out_files(int ifd, int ofd)
{
  int close_res;
  errno =0;
	if(ifd != 0)
	{
		close_res = close(ifd);
		if(close_res != 0)
		{
			if(debug == 1)
			{
                                sprintf(log_buf, "%s: error in closing the input-file, in: %s", progname.c_str(), "close_in_out_files()");
				log_str = log_buf;
				log(log_str);
			}
			else
			{
	  			fprintf(stderr, "%s: error in closing the input-file, in: %s\n", progname.c_str(), "close_in_out_files()");
			}
		}
	  ifd = 0;
	}
        if(ofd != 0)
	{
		close_res = close(ofd);
		if(close_res != 0)
		{
	  		if(debug == 1)
			{
                                sprintf(log_buf, "%s: error in closing the output-file/device, in: %s", progname.c_str(), "close_in_out_files()");
				log_str = log_buf;
				log(log_str);
			}
			else
			{
	  			fprintf(stderr, "%s: error in closing the output-file/device, in: %s\n", progname.c_str(), "close_in_out_files()");
			}
		}
	  ofd = 0;
	}
}


 int do_stat(const char *name, struct stat *sbuf)
 {
   int rc=0;

   errno =0;
    if(!to_stdout)
    {
	rc = lstat(name, sbuf);
    }
    else
    {
    	rc = stat(name, sbuf);
    }
   return rc;
 }

 int do_stat(int fd, struct stat *sbuf)
 {
   int rc=0;
   errno =0;
   rc = fstat(fd, sbuf);
   return rc;
 }


uInt seek_on_err(int ifd, size_t count)
 {
        uInt b_skip;
	off_t curr_pos;
	off_t new_pos;

        if(debug == 1)
	{
        	sprintf(log_buf, "%s: I/O error reading file/device, errno=%d; skipping %d bytes.", progname.c_str(), errno, count);
		log_str = log_buf;
		log(log_str);
	}
	else
	{
		fprintf(stderr, "%s: I/O error reading input file, errno=%d; trying to read next block: %d bytes.\n", progname.c_str(), errno, count);
	}

	errno = 0;


	curr_pos = lseek(ifd, 0, SEEK_CUR);


	new_pos = lseek(ifd, count, SEEK_CUR);




	b_skip = new_pos - curr_pos;
	return b_skip;
 }

 int treat_file(string iname)
 {
   int rc = 0;
   int fres = 0;
   int unzsplit_res = 0;
   int close_res = 0; 	// result of closing zipped file
   unsigned int chnk_begin_counter = 0;
   string ifn, infn; 	// name of input-file
   string ofn, outfn; 	// name of output-file
   string path_ofn; 	// constructed path-name of the output file
   string path_ifn; 	// constructed path-name of the input file
   string path_in; 	// input pathname
   string path_out;	// output path name
   vector<string>::iterator it; // iterator for out-put names in names_cont
   unsigned int first_chunk_past = 0;

   int key = 0;
   ifn = "";
   infn = "";
   ofn = "";
   outfn = "";
   path_ofn = "";
   path_ifn = "";
   path_in = "";
   path_out = "";

   	if(!strcmp(iname.c_str(), "-"))
	{
		from_stdin = 1;
		rc = treat_stdin();
	}
	else
	{

	   if(multivol == 0)
	   {

	   	if(!to_stdout)
		{
			if(out_filename == 0 && opath == 0 && ipath == 0)
			{


				path_in = "";
				path_out = "";

				outfn = OutputFilename;

				ofn = make_ofname(outfn);


				if(out_device == 1)
				{
                                	path_ofn = out_devn;
				}
				else
				{
					path_ofn = path_out + ofn;
				}
			}
			else if(out_filename == 1 && opath == 1 && ipath == 1)
			{

				path_in = in_pathn;

				outfn = out_filen;

				ofn = make_ofname(outfn);

				path_out = out_pathn;

				path_ofn = path_out + ofn;
			}
			else if(out_filename == 1 && opath == 1 && ipath == 0)
			{
				path_in = get_path(iname);

				outfn = out_filen;

				ofn = make_ofname(outfn);

				path_out = out_pathn;

				path_ofn = path_out + ofn;
			}
			else if(out_filename == 1 && opath == 0 && ipath == 0)
			{
				path_in = get_path(iname);

				path_out = "";

				outfn = out_filen;

				ofn = make_ofname(outfn);

				path_ofn = path_out + ofn;
			}
			else if(out_filename == 0 && opath == 1 && ipath == 0)
			{
				path_in = get_path(iname);

				outfn = OutputFilename;

				ofn = make_ofname(outfn);

				path_out = out_pathn;

				path_ofn = path_out + ofn;
			}
			else if(out_filename == 0 && opath == 0 && ipath == 1)
			{

				path_in = in_pathn;

				path_out = "";

				outfn = OutputFilename;

				ofn = make_ofname(outfn);

				if(out_device == 1)
				{
                                	path_ofn = out_devn;
				}
				else
				{

					path_ofn = path_out + ofn;
				}
			}
			else if(out_filename == 0 && opath == 1 && ipath == 1)
			{

				path_in = in_pathn;

				outfn = OutputFilename;

				ofn = make_ofname(outfn);

				path_out = out_pathn;

				path_ofn = path_out + ofn;
			}
			else if(out_filename == 1 && opath == 0 && ipath == 1)
			{

				path_in = in_pathn;

				path_out = "";

				outfn = out_filen;

				ofn = make_ofname(outfn);

				path_ofn = path_out + ofn;
			}
			else
			{
                 if(debug == 1)
				{
                    sprintf(log_buf, "%s: input parameters were not conform, iname: %s, in_pathn: %s, out_pathn: %s, in: %s", progname.c_str(), iname.c_str(), in_pathn.c_str(), out_pathn.c_str(), "treat_file()");
					log_str = log_buf;
					log(log_str);
				}
				else
				{
       					fprintf(stderr, "%s: input parameters were not conform, iname: %s, in_pathn: %s, out_pathn: %s, in: %s\n", progname.c_str(), iname.c_str(), in_pathn.c_str(), out_pathn.c_str(), "treat_file()");
				}
				rc = -1;
				return rc;
			}

			ofd = open(path_ofn.c_str(),O_RDWR | O_CREAT | O_LARGEFILE, S_IRUSR | S_IWUSR);


			if( ofd == -1)
			{
				if(debug == 1)
				{
                                        sprintf(log_buf, "%s: cannot open file/device: %s, in: %s", progname.c_str(), path_ofn.c_str(), "treat_file()");
					log_str = log_buf;
					log(log_str);
				}
				else
                                {
					fprintf (stderr, "%s: cannot open file/device: %s, in: %s\n", progname.c_str(), path_ofn.c_str(), "treat_file()");
				}
				rc = -1;
				return rc;
			}

		}
		else
		{

			if(ipath == 0)
			{

				path_in = get_path(iname);
			}
			if(ipath == 1)
			{

				path_in = in_pathn;
			}

			ofn.assign("stdout");
			if (do_stat(fileno(stdout), &istat) != 0)
			{
                                if(debug == 1)
				{
					sprintf(log_buf, "%s: cannot access %s in do_stat(), in: %s", progname.c_str(), ofn.c_str(), "treat_file()");
					log_str = log_buf;
					log(log_str);
				}
				else
				{
					fprintf (stderr, "%s: cannot access %s in do_stat(), in: %s\n", progname.c_str(), ofn.c_str(), "treat_file()");
				}
				rc = -1;
				return rc;
			}


			ofd = fileno(stdout);
		}

		fres = search_file_parts(iname, chnk_begin_counter);

		if(fres !=0 || names_cont.size() == 0)
		{
			if(fres == -1)
			{
                                if(debug == 1)
				{
					sprintf(log_buf, "%s: no files similar to pattern: %s were found, in: %s, return=%d, check the input directory", progname.c_str(), iname.c_str(), "search_file_parts()", fres);
					log_str = log_buf;
					log(log_str);
				}
				else
				{
					fprintf(stderr, "%s: no files similar to pattern: %s were found, in: %s, return=%d, check the input directory\n", progname.c_str(), iname.c_str(), "search_file_parts()", fres);
				}
				rc = -1;
				return rc;
			}
			else if(fres == -2)
			{
                                if(debug == 1)
				{
					sprintf(log_buf, "%s: can not find beginning part (*_%d.spl.zp) of splitted chunks, in: %s, return=%d, aborting operation", progname.c_str(), chnk_begin_counter, "search_file_parts()", fres);
					log_str = log_buf;
					log(log_str);
				}
				else
				{
					fprintf(stderr, "%s: can not find beginning part (*_%d.spl.zp) of splitted chunks, in: %s, return=%d, aborting operation\n", progname.c_str(), chnk_begin_counter, "search_file_parts()", fres);
				}
            			rc = -1;
				return rc;
			}
			else if(fres == -3)
			{
                                if(debug == 1)
				{
					sprintf(log_buf, "%s: inconsistency in file-chunks detected, in: %s, return=%d, aborting operation", progname.c_str(), "search_file_parts()", fres);
					log_str = log_buf;
					log(log_str);
				}
				else
				{
					fprintf(stderr, "%s: inconsistency in file-chunks detected, in: %s, return=%d, aborting operation\n", progname.c_str(), "search_file_parts()", fres);
				}
				rc = -1;
				return rc;
			}
			else
			{
                                if(debug == 1)
				{
					sprintf(log_buf, "%s: unknown error in: %s, return=%d, aborting operation", progname.c_str(), "search_file_parts()", fres);
					log_str = log_buf;
					log(log_str);
				}
				else
				{
					fprintf(stderr, "%s: unknown error in: %s, return=%d, aborting operation\n", progname.c_str(), "search_file_parts()", fres);
				}
				rc = -1;
				return rc;
			}
		}


		for (it=names_cont.begin(); it != names_cont.end(); it++ )
		{

			if(debug == 1)
			{
				sprintf(log_buf, "%s: inflating %s", progname.c_str(), (*it).c_str());
				log_str = log_buf;
				log(log_str);
			}
			else
			{
				fprintf (stderr, "%s: inflating %s\n", progname.c_str(), (*it).c_str());
			}

			path_ifn = path_in + *it;
				// check if input file is present and can be accessed
			if (do_stat(path_ifn.c_str(), &istat) != 0)
			{
				if(debug == 1)
				{
					sprintf(log_buf, "%s: error in accessing input file: %s, in: %s", progname.c_str(), path_ifn.c_str(), "treat_file()");
					log_str = log_buf;
					log(log_str);
				}
				else
				{
					fprintf (stderr, "%s: error in accessing input file: %s, in: %s\n", progname.c_str(), path_ifn.c_str(), "treat_file()");
				}
				rc = -1;
				return rc;
			}

			ifd = open(path_ifn.c_str(), O_RDONLY | O_LARGEFILE);

			if( ifd == -1)
			{
				if(debug == 1)
				{
					sprintf(log_buf, "%s: cannot open file/device: %s, in: %s", progname.c_str(), path_ifn.c_str(), "treat_file()");
					log_str = log_buf;
					log(log_str);
				}
				else
				{
					fprintf (stderr, "%s: cannot open file/device: %s, in: %s\n", progname.c_str(), path_ifn.c_str(), "treat_file()");
				}
				rc = -1;
				return rc;
			}


			if(!to_stdout)
			{
				unzsplit_res = do_unzsplit(ifd, ofd, path_ifn, path_ofn);
			}
			else
			{
				unzsplit_res = do_unzsplit(ifd, ofd, path_ifn);
			}

			if(unzsplit_res == 0)
			{
				close_res = close(ifd);
				ifd = 0;
				rc = 0;
				if(close_res != 0)
				{
					if(debug == 1)
					{
						sprintf(log_buf, "%s: error in closing the input file: %s, in: %s", progname.c_str(), path_ifn.c_str(), "treat_file()");
						log_str = log_buf;
						log(log_str);
					}
					else
					{
						fprintf (stderr, "%s: error in closing the input file: %s, in: %s\n", progname.c_str(), path_ifn.c_str(), "treat_file()");
					}
					close_res = close(ofd);
					if(close_res != 0)
					{
						if(debug == 1)
						{
							sprintf(log_buf, "%s: error in closing the output file: %s, in: %s return=%d", progname.c_str(), path_ofn.c_str(), "treat_file()", close_res);
							log_str = log_buf;
							log(log_str);
						}
						else
						{
							fprintf (stderr, "%s: error in closing the output file: %s, in: %s return=%d\n", progname.c_str(), path_ofn.c_str(), "treat_file()", close_res);
						}
					}
					ofd = 0;
					rc = -1;
					break;
				}
			}

			else if(unzsplit_res == -3)
			{
				if(debug == 1)
				{
					sprintf(log_buf, "%s: error in estimating the file size in do_stat(), in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
					log_str = log_buf;
					log(log_str);
				}
				else
				{
					fprintf (stderr, "%s: error in estimating the file size in do_stat(), in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
				}
				close_in_out_files(ifd, ofd);
				rc = -1;
				break;
			}
			else if(unzsplit_res == -4)
				{
                                        if(debug == 1)
					{
						sprintf(log_buf, "%s: error in accessing stdin, in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
						log_str = log_buf;
						log(log_str);
					}
					else
					{
						fprintf (stderr, "%s: error in accessing stdin, in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
					}
					close_in_out_files(ifd, ofd);
					rc = -1;
					break;
				}
			else if(unzsplit_res == -5)
			{
				if(debug == 1)
				{
					sprintf(log_buf, "%s: error in writing to output file, in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
					log_str = log_buf;
					log(log_str);
				}
				else
				{
					fprintf (stderr, "%s: error in writing to output file, in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
				}
				close_in_out_files(ifd, ofd);
				rc = -1;
				break;
			}
			else if(unzsplit_res == -6)
			{
				if(debug == 1)
				{
					sprintf(log_buf, "%s: error in initialization of inflation, in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
					log_str = log_buf;
					log(log_str);
				}
				else
				{
					fprintf (stderr, "%s: error in initialization of inflation, in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
				}
				close_in_out_files(ifd, ofd);
				rc = -1;
				break;
			}
			else if(unzsplit_res == -7)
			{
				if(debug == 1)
				{
					sprintf(log_buf, "%s: unknown error, in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
					log_str = log_buf;
					log(log_str);
				}
				else
				{
					fprintf (stderr, "%s: unknown error, in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
				}
				close_in_out_files(ifd, ofd);
				rc = -1;
				break;
			}
			else if(unzsplit_res == -8)
			{
				if(debug == 1)
				{
					sprintf(log_buf, "%s: the stream structure was inconsistent, in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
					log_str = log_buf;
					log(log_str);
				}
				else
				{
					fprintf (stderr, "%s: the stream structure was inconsistent, in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
				}
				close_in_out_files(ifd, ofd);
				rc = -1;
				break;
			}
			else if(unzsplit_res == -10)
			{
				if(debug == 1)
				{
					sprintf(log_buf, "%s: error in accessing stdout, in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
					log_str = log_buf;
					log(log_str);
				}
				else
				{
					fprintf (stderr, "%s: error in accessing stdout, in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
				}
				close_in_out_files(ifd, ofd);
				rc = -1;
				break;
			}
			else if(unzsplit_res == -11)
			{
				if(debug == 1)
				{
					sprintf(log_buf, "%s: there was not enough memory, in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
					log_str = log_buf;
					log(log_str);
				}
				else
				{
					fprintf (stderr, "%s: there was not enough memory, in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
				}
				close_in_out_files(ifd, ofd);
				rc = -1;
				break;
			}
			else if(unzsplit_res == -12)
			{
				if(debug == 1)
				{
					sprintf(log_buf, "%s: corrupted input data, in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
					log_str = log_buf;
					log(log_str);
				}
				else
				{
					fprintf (stderr, "%s: corrupted input data, in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
				}
				close_in_out_files(ifd, ofd);
				rc = -1;
				break;
			}

			else if(unzsplit_res == -13)
			{
				if(debug == 1)
				{
					sprintf(log_buf, "%s: I/O error occured, please try again with \"--noerror\" option; return=%d, in: %s", progname.c_str(), unzsplit_res, "do_unzsplit()");
					log_str = log_buf;
					log(log_str);
				}
				else
				{
					fprintf (stderr, "%s: I/O error occured, please try again with \"--noerror\" option; return=%d, in: %s\n", progname.c_str(), unzsplit_res, "do_unzsplit()");
				}
				close_in_out_files(ifd, ofd);
				rc = -1;
				break;
			}

			else
			{
				if(debug == 1)
				{
					sprintf(log_buf, "%s: unexpected error, in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
					log_str = log_buf;
					log(log_str);
				}
				else
				{
					fprintf (stderr, "%s: unexpected error, in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
				}
				close_in_out_files(ifd, ofd);
				rc = -1;
				break;
			}
		}

		  if(ofd != 0)
		  {
		 	close_res = close(ofd);
		 	ofd = 0;
		  }

	   }
	   else
	   {

		if(!to_stdout)
		{
			if(out_filename == 0 && opath == 0 && ipath == 0)
			{


				path_in = "";
				path_out = "";

				outfn = OutputFilename;

				ofn = make_ofname(outfn);


				if(out_device == 1)
				{
					path_ofn = out_devn;
				}
				else
				{
					path_ofn = path_out + ofn;
				}
			}
			else if(out_filename == 1 && opath == 1 && ipath == 1)
			{

				path_in = in_pathn;

				outfn = out_filen;

				ofn = make_ofname(outfn);

				path_out = out_pathn;

				path_ofn = path_out + ofn;
			}
			else if(out_filename == 1 && opath == 1 && ipath == 0)
			{
				path_in = get_path(iname);

				outfn = out_filen;

				ofn = make_ofname(outfn);

				path_out = out_pathn;

				path_ofn = path_out + ofn;
			}
			else if(out_filename == 1 && opath == 0 && ipath == 0)
			{
				path_in = get_path(iname);
					// assign NULL as output path
				path_out = "";

				outfn = out_filen;

				ofn = make_ofname(outfn);

				path_ofn = path_out + ofn;
			}
			else if(out_filename == 0 && opath == 1 && ipath == 0)
			{
				path_in = get_path(iname);
				outfn = OutputFilename;
				ofn = make_ofname(outfn);
				path_out = out_pathn;
				path_ofn = path_out + ofn;
			}
			else if(out_filename == 0 && opath == 0 && ipath == 1)
			{
				path_in = in_pathn;
				path_out = "";
				outfn = OutputFilename;
				ofn = make_ofname(outfn);

				if(out_device == 1)
				{
					path_ofn = out_devn;
				}
				else
				{
					path_ofn = path_out + ofn;
				}
			}
			else if(out_filename == 0 && opath == 1 && ipath == 1)
			{
				path_in = in_pathn;
				outfn = OutputFilename;
				ofn = make_ofname(outfn);
				path_out = out_pathn;
				path_ofn = path_out + ofn;
			}
			else if(out_filename == 1 && opath == 0 && ipath == 1)
			{
				path_in = in_pathn;
				path_out = "";
				outfn = out_filen;
				ofn = make_ofname(outfn);
				path_ofn = path_out + ofn;
			}
			else
			{
				if(debug == 1)
				{
					sprintf(log_buf, "%s: input parameters were not conform, iname: %s, in_pathn: %s, out_pathn: %s, in: %s", progname.c_str(), iname.c_str(), in_pathn.c_str(), out_pathn.c_str(), "treat_file()");
					log_str = log_buf;
					log(log_str);
				}
				else
				{
					fprintf(stderr, "%s: input parameters were not conform, iname: %s, in_pathn: %s, out_pathn: %s, in: %s\n", progname.c_str(), iname.c_str(), in_pathn.c_str(), out_pathn.c_str(), "treat_file()");
				}
				rc = -1;
				return rc;
			}

			ofd = open(path_ofn.c_str(),O_RDWR | O_CREAT | O_LARGEFILE, S_IRUSR | S_IWUSR);

			if( ofd == -1)
			{
				if(debug == 1)
				{
					sprintf(log_buf, "%s: cannot open file/device: %s, in: %s", progname.c_str(), path_ofn.c_str(), "treat_file()");
					log_str = log_buf;
					log(log_str);
				}
				else
				{
					fprintf (stderr, "%s: cannot open file/device: %s, in: %s\n", progname.c_str(), path_ofn.c_str(), "treat_file()");
				}
				rc = -1;
				return rc;
			}

		}
		else
		{
			if(ipath == 0)
			{
				path_in = get_path(iname);
			}
			if(ipath == 1)
			{
				path_in = in_pathn;
			}


			ofn.assign("stdout");
			if (do_stat(fileno(stdout), &istat) != 0)
			{
				if(debug == 1)
				{
					sprintf(log_buf, "%s: cannot access %s in do_stat(), in: %s", progname.c_str(), ofn.c_str(), "treat_file()");
					log_str = log_buf;
					log(log_str);
				}
				else
				{
					fprintf (stderr, "%s: cannot access %s in do_stat(), in: %s\n", progname.c_str(), ofn.c_str(), "treat_file()");
				}
				rc = -1;
				return rc;
			}


			ofd = fileno(stdout);

		}


		for(;;)
		{

			chnk_begin_counter = first_chunk;

			fres = search_file_parts(iname, chnk_begin_counter);

			while(fres !=0 || names_cont.size() == 0)
			{
				if(fres == -1)
				{
					if(debug == 1)
					{
						sprintf(log_buf, "%s: no files similar to pattern: %s were found, in: %s, return=%d, check the input directory", progname.c_str(), iname.c_str(), "search_file_parts()", fres);
						log_str = log_buf;
						log(log_str);
					}
					else
					{
						fprintf(stderr, "%s: no files similar to pattern: %s were found, in: %s, return=%d, check the input directory\n", progname.c_str(), iname.c_str(), "search_file_parts()", fres);
					}


					first_chunk = first_chunk_past;
				}
				else if(fres == -2)
				{
					if(debug == 1)
					{
						sprintf(log_buf, "%s: can not find beginning part (*_%d.spl.zp) of splitted chunks, in: %s, return=%d, aborting operation", progname.c_str(), chnk_begin_counter, "search_file_parts()", fres);
						log_str = log_buf;
						log(log_str);
					}
					else
					{
						fprintf(stderr, "%s: can not find beginning part (*_%d.spl.zp) of splitted chunks, in: %s, return=%d, aborting operation\n", progname.c_str(), chnk_begin_counter, "search_file_parts()", fres);
					}

				   first_chunk = first_chunk_past;
				}
				else if(fres == -3)
				{
					if(debug == 1)
					{
						sprintf(log_buf, "%s: inconsistency in file-chunks detected, in: %s, return=%d, aborting operation", progname.c_str(), "search_file_parts()", fres);
						log_str = log_buf;
						log(log_str);
					}
					else
					{
						fprintf(stderr, "%s: inconsistency in file-chunks detected, in: %s, return=%d, aborting operation\n", progname.c_str(), "search_file_parts()", fres);
					}

				   first_chunk = first_chunk_past;
				}
				else
				{
					if(debug == 1)
					{
						sprintf(log_buf, "%s: unknown error in: %s, return=%d, aborting operation", progname.c_str(), "search_file_parts()", fres);
						log_str = log_buf;
						log(log_str);
					}
					else
					{
						fprintf(stderr, "%s: unknown error in: %s, return=%d, aborting operation\n", progname.c_str(), "search_file_parts()", fres);
					}


				   first_chunk = first_chunk_past;
				}

				fprintf(stderr, "Please change volume and type (Y/y)es to retry or (N/n)o to end and exit [y]:");
				key = cin.get();

					if(key == 'Y' || key == 'y' || key == 'N' || key == 'n')
					{
						cin.get();
					}
				if(key == 'Y' || key == 'y' || key == '\n')
				{
					fprintf(stderr, "\n");


					chnk_begin_counter = first_chunk;

					fres = search_file_parts(iname, chnk_begin_counter);
					continue;
				}
				else if (key == 'N' || key == 'n')
				{
					fprintf(stderr, "\n");
					fprintf(stderr, "User requested abort of operation.\n");

					close_res = close(ofd);
					if(close_res != 0)
					{
						if(debug == 1)
						{
							sprintf(log_buf, "%s: error in closing the output file: %s, in: %s return=%d", progname.c_str(), path_ofn.c_str(), "treat_file()", close_res);
							log_str = log_buf;
							log(log_str);
						}
						else
						{
							fprintf (stderr, "%s: error in closing the output file: %s, in: %s return=%d\n", progname.c_str(), path_ofn.c_str(), "treat_file()", close_res);
						}
					}
					ofd = 0;
					rc = -1;
					return rc;
				}
				else
				{
					fprintf(stderr, "\n");
					fprintf(stderr, "Not recognized character, abort of operation...\n");

					close_res = close(ofd);
					if(close_res != 0)
					{
						if(debug == 1)
						{
							sprintf(log_buf, "%s: error in closing the output file: %s, in: %s return=%d", progname.c_str(), path_ofn.c_str(), "treat_file()", close_res);
							log_str = log_buf;
							log(log_str);
						}
						else
						{
							fprintf (stderr, "%s: error in closing the output file: %s, in: %s return=%d\n", progname.c_str(), path_ofn.c_str(), "treat_file()", close_res);
						}
					}
					ofd = 0;
					rc = -1;
					return rc;
				}
			}

			for (it=names_cont.begin(); it != names_cont.end(); it++ )
			{

				if(debug == 1)
				{
					sprintf(log_buf, "%s: inflating %s", progname.c_str(), (*it).c_str());
					log_str = log_buf;
					log(log_str);
				}
				else
				{
					fprintf (stderr, "%s: inflating %s\n", progname.c_str(), (*it).c_str());
				}

				path_ifn = path_in + *it;
				if (do_stat(path_ifn.c_str(), &istat) != 0)
				{
					if(debug == 1)
					{
						sprintf(log_buf, "%s: error in accessing input file: %s, in: %s", progname.c_str(), path_ifn.c_str(), "treat_file()");
						log_str = log_buf;
						log(log_str);
					}
					else
					{
						fprintf (stderr, "%s: error in accessing input file: %s, in: %s\n", progname.c_str(), path_ifn.c_str(), "treat_file()");
					}

					close_res = close(ofd);
					if(close_res != 0)
					{
						if(debug == 1)
						{
							sprintf(log_buf, "%s: error in closing the output file: %s, in: %s return=%d", progname.c_str(), path_ofn.c_str(), "treat_file()", close_res);
							log_str = log_buf;
							log(log_str);
						}
						else
						{
							fprintf (stderr, "%s: error in closing the output file: %s, in: %s return=%d\n", progname.c_str(), path_ofn.c_str(), "treat_file()", close_res);
						}
					}
					ofd = 0;
					rc = -1;
					return rc;
				}

				ifd = open(path_ifn.c_str(), O_RDONLY | O_LARGEFILE);

				if( ifd == -1)
				{
					if(debug == 1)
					{
						sprintf(log_buf, "%s: cannot open file/device: %s, in: %s", progname.c_str(), path_ifn.c_str(), "treat_file()");
						log_str = log_buf;
						log(log_str);
					}
					else
					{
						fprintf (stderr, "%s: cannot open file/device: %s, in: %s\n", progname.c_str(), path_ifn.c_str(), "treat_file()");
					}

					close_res = close(ofd);
					if(close_res != 0)
					{
						if(debug == 1)
						{
							sprintf(log_buf, "%s: error in closing the output file: %s, in: %s return=%d", progname.c_str(), path_ofn.c_str(), "treat_file()", close_res);
							log_str = log_buf;
							log(log_str);
						}
						else
						{
							fprintf (stderr, "%s: error in closing the output file: %s, in: %s return=%d\n", progname.c_str(), path_ofn.c_str(), "treat_file()", close_res);
						}
					}
					ofd = 0;
					rc = -1;
					return rc;
				}


				if(!to_stdout)
				{
					unzsplit_res = do_unzsplit(ifd, ofd, path_ifn, path_ofn);
				}
				else
				{
					unzsplit_res = do_unzsplit(ifd, ofd, path_ifn);
				}

				if(unzsplit_res == 0)
				{
					close_res = close(ifd);
					ifd = 0;
					rc = 0;
					if(close_res != 0)
					{
						if(debug == 1)
						{
							sprintf(log_buf, "%s: error in closing the input file: %s, in: %s", progname.c_str(), path_ifn.c_str(), "treat_file()");
							log_str = log_buf;
							log(log_str);
						}
						else
						{
							fprintf (stderr, "%s: error in closing the input file: %s, in: %s\n", progname.c_str(), path_ifn.c_str(), "treat_file()");
						}
						close_res = close(ofd);
						if(close_res != 0)
						{
							if(debug == 1)
							{
								sprintf(log_buf, "%s: error in closing the output file: %s, in: %s return=%d", progname.c_str(), path_ofn.c_str(), "treat_file()", close_res);
								log_str = log_buf;
								log(log_str);
							}
							else
							{
								fprintf (stderr, "%s: error in closing the output file: %s, in: %s return=%d\n", progname.c_str(), path_ofn.c_str(), "treat_file()", close_res);
							}
						}
						ofd = 0;
						rc = -1;
						return rc;
						//break;
					}
					// save the current value of begin counter to its previous_value
					first_chunk_past = first_chunk;
				}

				else if(unzsplit_res == -3)
				{
					if(debug == 1)
					{
						sprintf(log_buf, "%s: error in estimating the file size in do_stat(), in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
						log_str = log_buf;
						log(log_str);
					}
					else
					{
						fprintf (stderr, "%s: error in estimating the file size in do_stat(), in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
					}
					close_in_out_files(ifd, ofd);
					rc = -1;
					return rc;
					//break;
				}
				else if(unzsplit_res == -4)
				{
					if(debug == 1)
					{
					sprintf(log_buf, "%s: error in accessing stdin, in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
					log_str = log_buf;
					log(log_str);
					}
					else
					{
					fprintf (stderr, "%s: error in accessing stdin, in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
					}
					close_in_out_files(ifd, ofd);
					rc = -1;
					return rc;
					//break;
				}
				else if(unzsplit_res == -5)
				{
					if(debug == 1)
					{
						sprintf(log_buf, "%s: error in writing to output file, in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
						log_str = log_buf;
						log(log_str);
					}
					else
					{
						fprintf (stderr, "%s: error in writing to output file, in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
					}
					close_in_out_files(ifd, ofd);
					rc = -1;
					return rc;
					//break;
				}
				else if(unzsplit_res == -6)
				{
					if(debug == 1)
					{
						sprintf(log_buf, "%s: error in initialization of inflation, in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
						log_str = log_buf;
						log(log_str);
					}
					else
					{
						fprintf (stderr, "%s: error in initialization of inflation, in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
					}
					close_in_out_files(ifd, ofd);
					rc = -1;
					return rc;
					//break;
				}
				else if(unzsplit_res == -7)
				{
					if(debug == 1)
					{
						sprintf(log_buf, "%s: unknown error in inflate(), in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
						log_str = log_buf;
						log(log_str);
					}
					else
					{
						fprintf (stderr, "%s: unknown error in inflate(), in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
					}
					close_in_out_files(ifd, ofd);
					rc = -1;
					return rc;
					//break;
				}
				else if(unzsplit_res == -8)
				{
					if(debug == 1)
					{
						sprintf(log_buf, "%s: the stream structure was inconsistent, in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
						log_str = log_buf;
						log(log_str);
					}
					else
					{
						fprintf (stderr, "%s: the stream structure was inconsistent, in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
					}
					close_in_out_files(ifd, ofd);
					rc = -1;
					return rc;
					//break;
				}
				else if(unzsplit_res == -10)
				{
					if(debug == 1)
					{
						sprintf(log_buf, "%s: error in accessing stdout, in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
						log_str = log_buf;
						log(log_str);
					}
					else
					{
						fprintf (stderr, "%s: error in accessing stdout, in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
					}
					close_in_out_files(ifd, ofd);
					rc = -1;
					return rc;
					//break;
				}
				else if(unzsplit_res == -11)
				{
					if(debug == 1)
					{
						sprintf(log_buf, "%s: not enough memory, in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
						log_str = log_buf;
						log(log_str);
					}
					else
					{
						fprintf (stderr, "%s: not enough memory, in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
					}
					close_in_out_files(ifd, ofd);
					rc = -1;
					return rc;
					//break;
				}
				else if(unzsplit_res == -12)
				{
					if(debug == 1)
					{
						sprintf(log_buf, "%s: corrupted input data, in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
						log_str = log_buf;
						log(log_str);
					}
					else
					{
						fprintf (stderr, "%s: corrupted input data, in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
					}
					close_in_out_files(ifd, ofd);
					rc = -1;
					return rc;
					//break;
				}

				else if(unzsplit_res == -13)
				{
					if(debug == 1)
					{
						sprintf(log_buf, "%s: I/O error occured, please try again with \"--noerror\" option; return=%d, in: %s", progname.c_str(), unzsplit_res, "do_unzsplit()");
						log_str = log_buf;
						log(log_str);
					}
					else
					{
						fprintf (stderr, "%s: I/O error occured, please try again with \"--noerror\" option; return=%d, in: %s\n", progname.c_str(), unzsplit_res, "do_unzsplit()");
					}
					close_in_out_files(ifd, ofd);
					rc = -1;
					return rc;
				}

				else
				{
					if(debug == 1)
					{
						sprintf(log_buf, "%s: unexpected error, in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
						log_str = log_buf;
						log(log_str);
					}
					else
					{
						fprintf (stderr, "%s: unexpected error, in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
					}
					close_in_out_files(ifd, ofd);
					rc = -1;
					return rc;
					//break;
				}
			}


			fprintf(stderr, "Would you like to read next volume?\nPlease change volume and type (Y/y)es to start or (N/n)o to end and exit [y]:");
			key = cin.get();

				if(key == 'Y' || key == 'y' || key == 'N' || key == 'n')
				{
					cin.get();
				}
			if(key == 'Y' || key == 'y' || key == '\n')
			{
				fprintf(stderr, "\n");

				continue;
			}
			else if (key == 'N' || key == 'n')
			{
				fprintf(stderr, "\n");
				fprintf(stderr, "User requested abort of operation.\n");

				close_in_out_files(ifd, ofd);
				break;
			}
			else
			{
				fprintf(stderr, "\n");
				fprintf(stderr, "Not recognized character, abort of operation.\n");

				close_in_out_files(ifd, ofd);
				break;
			}
		}

	   }

	}


  total_time_elps = elaps_time(end_sum_elps_time);

  av_speed_s = av_speed(end_sum_elps_time, end_sum_written);

  if(debug == 1)
  {
  	sprintf(log_buf, "%s: total sum of bytes read: %lld; written: %lld; elapsed time: %s", progname.c_str(), end_sum_read, end_sum_written, total_time_elps.c_str());
	log_str = log_buf;
	log(log_str);

	sprintf(log_buf, "%s: average speed of restoring: %s", progname.c_str(), av_speed_s.c_str());
	log_str = log_buf;
	log(log_str);
  }
  else
  {
  	fprintf (stderr, "%s: total sum of bytes read: %lld; written: %lld; elapsed time: %s\n", progname.c_str(), end_sum_read, end_sum_written, total_time_elps.c_str());
	fprintf (stderr, "%s: average speed of restoring: %s\n", progname.c_str(), av_speed_s.c_str());
  }
  return rc;
 }

int do_unzsplit(int ifd, int ofd, string path_ifn, string path_ofn)
{
 int res=0;
 int count=0;  // the size of the 1024 bytes buffer after compression
 z_stream z;
 Bytef input_buffer[io_block_size];
 Bytef output_buffer[io_block_size];
 int status = Z_OK;
 size_t read_res = 0;	// result of read operation could be SAFE_READ_ERROR (-1)
 struct stat ostat;         	/* status for output file */
 ssize_t testwr = 0;
 string path_bad_file;
 unsigned long long sum_read = 0;	// the ammount of read bytes
 unsigned long long sum_written = 0; // the amount of written bytes

 unsigned long sum_elps_time = 0;  	// elapsed time in seconds
 time_t start_sec, end_sec; 	// begin time im seconds, end time in seconds
 string time_elps; 		// elapsed time as formatted string


    z.zalloc = (alloc_func)0;
	z.zfree = (free_func)0;
	z.opaque = (voidpf)0;

	z.avail_in = 0;
	z.next_out = output_buffer;
	z.avail_out = io_block_size;

	if(inflateInit2(&z, 15) != Z_OK)
	{
		res = -6;
		return res;
	}

	time(&start_sec);

	for ( ; ; )
	{
		if (z.avail_in == 0)
		{
			if(noerror == 1)
			{
				memset(input_buffer, '\0', io_block_size);
			}
		  	z.next_in = input_buffer;
		  	read_res = safe_read(ifd, input_buffer, io_block_size);
			if(read_res == SAFE_READ_ERROR)
			{
				z.avail_in = 0;
			}
			else
			{
                               z.avail_in = read_res;
			}

			if(read_res == SAFE_READ_ERROR && noerror == 1)
			{
				z.avail_in = seek_on_err(ifd, safe_seek_block);
			}
		  	sum_read += z.avail_in;
		}

		if (z.avail_in == 0 && read_res == SAFE_READ_ERROR)
		{
		  if(noerror == 0)
		  {
			z.next_out = output_buffer;
			z.avail_out = io_block_size;
			res = -13;


			inflateEnd(&z);


				time(&end_sec);


				sum_elps_time = (unsigned long)end_sec - (unsigned long)start_sec;

				time_elps = elaps_time(sum_elps_time);


				end_sum_read += sum_read;
				end_sum_written += sum_written;
				end_sum_elps_time += sum_elps_time;
			return res;
		  }
		}
		else if (z.avail_in == 0 && read_res != SAFE_READ_ERROR)
		{
		  z.next_out = output_buffer;
		  z.avail_out = io_block_size;
		  res = 0;
		  break;
		}

		status = inflate(&z, Z_NO_FLUSH);

		if (status != Z_OK)
		{
			if(status == Z_STREAM_END)
			{
				res = 0;
				count = io_block_size - z.avail_out;
				if(count)
				{
			  		testwr = safe_write(ofd, output_buffer, count);
					sum_written += testwr;

					if(testwr != count)
		  	  		{
    		  				res = -5;
    		  				break;
		  	  		}
					z.next_out = output_buffer;
					z.avail_out = io_block_size;
				}

				break;
			}
			if(status == Z_DATA_ERROR)
			{
                do
				{
                     count = io_block_size -z.avail_in;
                 	if(count)
					{
						if(noerror == 1)
						{
							memset(input_buffer, '\0', io_block_size);
						}
						z.next_in = input_buffer;
		  				read_res = safe_read(ifd, input_buffer, count);
						if(read_res == SAFE_READ_ERROR)
						{
							z.avail_in = 0;
						}
						else
						{
                               				z.avail_in = read_res;
						}
						if(read_res == SAFE_READ_ERROR && noerror == 0)
						{
							z.next_out = output_buffer;
							z.avail_out = io_block_size;
							res = -13;


							inflateEnd(&z);


							time(&end_sec);


							sum_elps_time = (unsigned long)end_sec - (unsigned long)start_sec;

							time_elps = elaps_time(sum_elps_time);


							end_sum_read += sum_read;
							end_sum_written += sum_written;
							end_sum_elps_time += sum_elps_time;
							return res;
						}

						else if(read_res == SAFE_READ_ERROR && noerror == 1)
						{
							z.avail_in = seek_on_err(ifd, safe_seek_block);
						}
					   sum_read += z.avail_in;
					}
					status = inflateSync(&z);
				}
				while(status != Z_OK && status != Z_BUF_ERROR);
			}
			else if(status == Z_STREAM_ERROR)
			{
				res = -8;
				return res;
			}
			else if(status == Z_MEM_ERROR)
			{
				res = -11;
				return res;
			}
			else
			{
		  		res = -7;
		  		return res;
			}
		}

		count = io_block_size - z.avail_out;
		if(count)
		{
			  testwr = safe_write(ofd, output_buffer, count);
			  sum_written += testwr;
              		  if(testwr != count)
		  	  {
    		  		res = -5;
    		  		return res;
		  	  }
		}

    		if (do_stat(path_ofn.data(), &ostat) != 0)
		{
    		  res = -3;
    		  return res;
		}

     		z.next_out = output_buffer;
		z.avail_out = io_block_size;
	}
	if(status != Z_BUF_ERROR)
	{
		for ( ; ; )
		{
			status = inflate(&z, Z_FINISH);
			if(status == Z_OK)
			{
				do
				{
					count = io_block_size -z.avail_out;
					if(count)
					{
						testwr = safe_write(ofd, output_buffer, count);
						sum_written += testwr;
						if(testwr != count)
						{
							res = -5;
							break;
						}
						z.next_out = output_buffer;
						z.avail_out = io_block_size;
					}

					status = deflate(&z, Z_FINISH);

				}while (status == Z_OK);


				if(status == Z_STREAM_END)
				{
					count = io_block_size - z.avail_out;

					if(count)
					{
						testwr = safe_write(ofd, output_buffer, count);
						sum_written += testwr;
						if(testwr != count)
						{
							res = -5;
							break;
						}
					}
				}
			break;
			}
			else if(status == Z_STREAM_END)
			{
				count = io_block_size - z.avail_out;

				if(count)
				{
					testwr = safe_write(ofd, output_buffer, count);
					sum_written += testwr;
					if(testwr != count)
					{
						res = -5;
						break;
					}
				}

			break;
			}
			else if (status == Z_STREAM_ERROR)
			{
				res = -8;
				break;
			}
			else if (status == Z_DATA_ERROR)
			{
				res = -12;
				break;
			}
			else if(status == Z_BUF_ERROR)
			{
				break;
			}
		}
	}
 inflateEnd(&z);

 time(&end_sec);

 sum_elps_time = (unsigned long)end_sec - (unsigned long)start_sec;

 time_elps = elaps_time(sum_elps_time);


  if(debug == 1)
  {
        sprintf(log_buf, "%s: bytes read from %s: %lld; written to %s: %lld; elapsed time: %s", progname.c_str(), path_ifn.c_str(), sum_read, path_ofn.c_str(), sum_written, time_elps.c_str());
	log_str = log_buf;
	log(log_str);
  }
  else
  {
  	fprintf (stderr, "%s: bytes read from %s: %lld; written to %s: %lld; elapsed time: %s\n", progname.c_str(), path_ifn.c_str(), sum_read, path_ofn.c_str(), sum_written, time_elps.c_str());
  }

  end_sum_read += sum_read;
  end_sum_written += sum_written;
  end_sum_elps_time += sum_elps_time;

 return res;
}

int do_unzsplit(int ifd, int ofd, string path_ifn)
{
 int res=0;
 int count=0;
 z_stream z;
 Bytef input_buffer[io_block_size];
 Bytef output_buffer[io_block_size];
 int status = Z_OK;
 size_t read_res=0;
 struct stat ostat;         	/* status for output file */
 ssize_t testwr = 0;
 string path_bad_file;
 unsigned long long sum_read = 0;	// the ammount of read bytes
 unsigned long long sum_written = 0; // the amount of written bytes
 string logstr;
 string path_ofn;
 path_ofn.assign("stdout");

 unsigned long sum_elps_time = 0;  	// elapsed time in seconds
 time_t start_sec, end_sec; 	// begin time im seconds, end time in seconds
 string time_elps; 		// elapsed time as formatted string



 	z.zalloc = (alloc_func)0;
	z.zfree = (free_func)0;
	z.opaque = (voidpf)0;

	z.avail_in = 0;
	z.next_out = output_buffer;
	z.avail_out = io_block_size;


	if(inflateInit2(&z, 15) != Z_OK)
	{
		res = -6;
		return res;
	}

	time(&start_sec);

	for ( ; ; )
	{
		if (z.avail_in == 0)
		{
			if(noerror == 1)
		  	{
				memset(input_buffer, '\0', io_block_size);
		  	}
		  	z.next_in = input_buffer;
		  	read_res = safe_read(ifd, input_buffer, io_block_size);
			if(read_res == SAFE_READ_ERROR)
		  	{
				z.avail_in = 0;
		  	}
		  	else
		  	{
				z.avail_in = read_res;
		  	}

		  	if(read_res == SAFE_READ_ERROR && noerror == 1)
			{
				z.avail_in = seek_on_err(ifd, safe_seek_block);
			}
		  	sum_read += z.avail_in;
		}
		if (z.avail_in == 0 && read_res == SAFE_READ_ERROR)
		{

		  if(noerror == 0)
		  {
			z.next_out = output_buffer;
			z.avail_out = io_block_size;
			res = -13;


			inflateEnd(&z);


				time(&end_sec);


				sum_elps_time = (unsigned long)end_sec - (unsigned long)start_sec;

				time_elps = elaps_time(sum_elps_time);


				end_sum_read += sum_read;
				end_sum_written += sum_written;
				end_sum_elps_time += sum_elps_time;
			return res;
		  }
		}
		else if (z.avail_in == 0 && read_res != SAFE_READ_ERROR)
		{
		  z.next_out = output_buffer;
		  z.avail_out = io_block_size;
		  res = 0;
		  break;
		}

		status = inflate(&z, Z_NO_FLUSH);

		if (status != Z_OK)
		{
		  if(status == Z_STREAM_END)
			{
				res = 0;
				count = io_block_size - z.avail_out;
				if(count)
				{
			  		testwr = safe_write(ofd, output_buffer, count);
					sum_written += testwr;

					if(testwr != count)
		  	  		{
    		  				res = -5;
    		  				break;
		  	  		}
					z.next_out = output_buffer;
					z.avail_out = io_block_size;
				}

				break;

			}
			if(status == Z_DATA_ERROR)
			{
                do
				{
                    count = io_block_size -z.avail_in;
                 	if(count)
					{
						if(noerror == 1)
						{
							memset(input_buffer, '\0', io_block_size);
						}
						z.next_in = input_buffer;
		  				read_res = safe_read(ifd, input_buffer, count);
						if(read_res == SAFE_READ_ERROR)
						{
							z.avail_in = 0;
						}
						else
						{
                               				z.avail_in = read_res;
						}
						if(read_res == SAFE_READ_ERROR && noerror == 0)
						{
							z.next_out = output_buffer;
							z.avail_out = io_block_size;
							res = -13;


							inflateEnd(&z);


							time(&end_sec);


							sum_elps_time = (unsigned long)end_sec - (unsigned long)start_sec;

							time_elps = elaps_time(sum_elps_time);


							end_sum_read += sum_read;
							end_sum_written += sum_written;
							end_sum_elps_time += sum_elps_time;

							return res;
						}

						else if(read_res == SAFE_READ_ERROR && noerror == 1)
						{
							z.avail_in = seek_on_err(ifd, safe_seek_block);
						}
					   sum_read += z.avail_in;
					}
					status = inflateSync(&z);
				}
				while(status != Z_OK && status != Z_BUF_ERROR);

			}
			else if(status == Z_STREAM_ERROR)
			{
				res = -8;
				return res;
			}
			else if(status == Z_MEM_ERROR)
			{
				res = -11;
				return res;
			}
			else
			{
		  		res = -7;
		  		return res;
			}
		}

		count = io_block_size - z.avail_out;
		if(count)
		{
		  	testwr = safe_write(ofd, output_buffer, count);
			sum_written += testwr;
              		  if(testwr != count)
		  	  {
    		  		res = -5;
    		  		return res;
		  	  }
		}

    		if (do_stat(fileno(stdout), &ostat) != 0)
		{
    		  res = -10;
    		  break;
		}
		if(from_stdin)
		{
    			if (do_stat(fileno(stdin), &istat) != 0)
			{
    		  		res = -4;
    		  		break;
			}
		}

		z.next_out = output_buffer;
		z.avail_out = io_block_size;
	}
	if(status != Z_BUF_ERROR)
	{
		for ( ; ; )
		{
			status = inflate(&z, Z_FINISH);
			if(status == Z_OK)
			{
				do
				{
					count = io_block_size -z.avail_out;
					if(count)
					{
						testwr = safe_write(ofd, output_buffer, count);
						sum_written += testwr;
						if(testwr != count)
						{
							res = -5;
							break;
						}
						z.next_out = output_buffer;
						z.avail_out = io_block_size;
					}

					status = deflate(&z, Z_FINISH);

				}while (status == Z_OK);


				if(status == Z_STREAM_END)
				{
					count = io_block_size - z.avail_out;

					if(count)
					{
						testwr = safe_write(ofd, output_buffer, count);
						sum_written += testwr;
						if(testwr != count)
						{
							res = -5;
							break;
						}
					}
				}
			break;
			}
			else if(status == Z_STREAM_END)
			{
				count = io_block_size - z.avail_out;

				if(count)
				{
					testwr = safe_write(ofd, output_buffer, count);
					sum_written += testwr;
					if(testwr != count)
					{
						res = -5;
						break;
					}
				}

			break;
			}
			else if (status == Z_STREAM_ERROR)
			{
				res = -8;
				break;
			}
			else if (status == Z_DATA_ERROR)
			{
				res = -12;
				break;
			}
			else if(status == Z_BUF_ERROR)
			{
				break;
			}
		}
	}
 inflateEnd(&z);

 time(&end_sec);

 sum_elps_time = (unsigned long)end_sec - (unsigned long)start_sec;
 time_elps = elaps_time(sum_elps_time);


  if(debug == 1)
  {
        sprintf(log_buf, "%s: bytes read from %s: %lld; written to %s: %lld; elapsed time: %s", progname.c_str(), path_ifn.c_str(), sum_read, path_ofn.c_str(), sum_written, time_elps.c_str());
	log_str = log_buf;
	log(log_str);
  }
  else
  {
  	fprintf (stderr, "%s: bytes read from %s: %lld; written to %s: %lld; elapsed time: %s\n", progname.c_str(), sum_read, path_ifn.c_str(), sum_written, path_ofn.c_str(), time_elps.c_str());
  }

  end_sum_read += sum_read;
  end_sum_written += sum_written;
  end_sum_elps_time += sum_elps_time;

 return res;
}

int treat_stdin()
{
  int res = 0;
  int rc = 0; // return value for the main function
  string ofn, outfn; 		// name of output-file

  string path_ofn;
  string path_ifn;	// name of input path-filename, in this case this is stdin
  string path_out;	// output path
  int unzsplit_res;	// return result of the zsplit function
  int close_res; 	// result of closing zipped file





  if(multivol == 1)
  {
       if(debug == 1)
	{
		sprintf(log_buf, "%s: multivolume feature selected, cannot do it from stdin, not implemented yet, please try again without \'-m\' option, in: %s", progname.c_str(), "treat_stdin()");
		log_str = log_buf;
		log(log_str);
	}
	else
	{
		fprintf (stderr, "%s: multivolume feature selected, cannot do it from stdin, not implemented yet, please try again without \'-m\' option, in: %s\n", progname.c_str(), "treat_stdin()");
	}
	res = -1;
	return res;
  }

  path_ifn.assign("stdin");

  if (do_stat(fileno(stdin), &istat) != 0)
  {

        if(debug == 1)
	{
		sprintf(log_buf, "%s: cannot access input file: %s, in: %s", progname.c_str(), path_ifn.c_str(), "treat_stdin()");
		log_str = log_buf;
		log(log_str);
	}
	else
	{
		fprintf (stderr, "%s: cannot access input file: %s, in: %s\n", progname.c_str(), path_ifn.c_str(), "treat_stdin()");
	}
	res = -1;
	return res;
  }

  ifd = fileno(stdin);

  if(to_stdout)
  {
  	path_ofn.assign("stdout");
        if (do_stat(fileno(stdout), &istat) != 0)
	{
                if(debug == 1)
		{
			sprintf(log_buf, "%s: cannot access output file: %s, in: %s", progname.c_str(), path_ofn.c_str(), "treat_stdin()");
			log_str = log_buf;
			log(log_str);
		}
		else
		{
			fprintf (stderr, "%s: cannot access output file: %s, in: %s\n", progname.c_str(), path_ofn.c_str(), "treat_stdin()");
		}
		res = -1;
		return res;
	}

	ofd = fileno(stdout);

	for ( ; ; )
  	{
		unzsplit_res = do_unzsplit(ifd, ofd, path_ifn);

		if(unzsplit_res == 0)
		{
			close_in_out_files(ifd, ofd);
			res = 0;
			break;
		}
  		else if(unzsplit_res == -4)
		{
                        if(debug == 1)
			{
				sprintf(log_buf, "%s: error in accessing stdin, in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
				log_str = log_buf;
				log(log_str);
			}
			else
			{
				fprintf (stderr, "%s: error in accessing stdin, in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
			}
			close_in_out_files(ifd, ofd);
			res = -1;
			break;
		}
		else if(unzsplit_res == -5)
		{
                        if(debug == 1)
			{
				sprintf(log_buf, "%s: error in writing to output file, in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
				log_str = log_buf;
				log(log_str);
			}
			else
			{
				fprintf (stderr, "%s: error in writing to output file, in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
			}
			close_in_out_files(ifd, ofd);
			res = -1;
			break;
		}
		else if(unzsplit_res == -6)
		{
                        if(debug == 1)
			{
				sprintf(log_buf, "%s: error in initialization of inflation, in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
				log_str = log_buf;
				log(log_str);
			}
			else
			{
				fprintf (stderr, "%s: error in initialization of inflation, in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
			}
			close_in_out_files(ifd, ofd);
			res = -1;
			break;
		}

		else if(unzsplit_res == -7)
		{
                        if(debug == 1)
			{
				sprintf(log_buf, "%s: unknown error in inflate(), in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
				log_str = log_buf;
				log(log_str);
			}
			else
			{
				fprintf (stderr, "%s: unknown error in inflate(), in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
			}
			close_in_out_files(ifd, ofd);
			res = -1;
			break;
		}
		else if(unzsplit_res == -8)
		{
                        if(debug == 1)
			{
				sprintf(log_buf, "%s: stream structure was inconsistent, in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
				log_str = log_buf;
				log(log_str);
			}
			else
			{
				fprintf (stderr, "%s: stream structure was inconsistent, in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
			}
			close_in_out_files(ifd, ofd);
			res = -1;
			break;
		}
		else if(unzsplit_res == -10)
		{
                        if(debug == 1)
			{
				sprintf(log_buf, "%s: error in accessing stdout, in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
				log_str = log_buf;
				log(log_str);
			}
			else
			{
				fprintf (stderr, "%s: error in accessing stdout, in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
			}
			close_in_out_files(ifd, ofd);
			res = -1;
			break;
		}
		else if(unzsplit_res == -11)
		{
                        if(debug == 1)
			{
				sprintf(log_buf, "%s: there was not enough memory, in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
				log_str = log_buf;
				log(log_str);
			}
			else
			{
				fprintf (stderr, "%s: there was not enough memory, in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
			}
			close_in_out_files(ifd, ofd);
			res = -1;
			break;
		}
		else if(unzsplit_res == -12)
		{
                        if(debug == 1)
			{
				sprintf(log_buf, "%s: corrupted input data, in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
				log_str = log_buf;
				log(log_str);
			}
			else
			{
				fprintf (stderr, "%s: corrupted input data, in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
			}
			close_in_out_files(ifd, ofd);
			res = -1;
			break;
		}

		else if(unzsplit_res == -13)
		{
			if(debug == 1)
			{
				sprintf(log_buf, "%s: I/O error occured, please try again with \"--noerror\" option; return=%d, in: %s", progname.c_str(), unzsplit_res, "do_unzsplit()");
				log_str = log_buf;
				log(log_str);
			}
			else
			{
				fprintf (stderr, "%s: I/O error occured, please try again with \"--noerror\" option; return=%d, in: %s\n", progname.c_str(), unzsplit_res, "do_unzsplit()");
			}
			close_in_out_files(ifd, ofd);
			res = -1;
			break;
		}

		else
		{
                        if(debug == 1)
			{
				sprintf(log_buf, "%s: unexpected error..., in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
				log_str = log_buf;
				log(log_str);
			}
			else
			{
				fprintf (stderr, "%s: unexpected error..., in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
			}
			close_in_out_files(ifd, ofd);
			res = -1;
			break;
		}
	}
  }
  else
  {
  	if(out_filename == 0 && opath == 0)
	{
		outfn = get_def_filename();
		ofn = make_ofname(outfn);
		path_out = "";
		if(out_device == 1)
		{
			path_ofn = out_devn;
		}
		else
		{
			path_ofn = path_out + ofn;
		}
	}
	else if(out_filename == 1 && opath == 1)
	{
		outfn = out_filen;
		ofn = make_ofname(outfn);
		path_out = out_pathn;
		path_ofn = path_out + ofn;
	}
	else if(out_filename == 1 && opath == 0)
	{
		outfn = out_filen;
		ofn = make_ofname(outfn);
		path_out = "";
		path_ofn = path_out + ofn;
	}
	else if(out_filename == 0 && opath == 1)
	{
		outfn = get_def_filename();
		ofn = make_ofname(outfn);
		path_out = out_pathn;
		path_ofn = path_out + ofn;
	}
	else
	{
		if(debug == 1)
		{
                	sprintf(log_buf, "%s: input parameters were not conform: path_ifn: %s, ofn: %s, out_pathn: %s, path_ofn: %s, in: %s", progname.c_str(), path_ifn.c_str(), ofn.c_str(), out_pathn.c_str(), path_ofn.c_str(), "treat_stdin()");
			log_str = log_buf;
			log(log_str);
		}
		else
		{
			fprintf(stderr, "%s: input parameters were not conform: path_ifn: %s, ofn: %s, out_pathn: %s, path_ofn: %s, in: %s\n", progname.c_str(), path_ifn.c_str(), ofn.c_str(), out_pathn.c_str(), path_ofn.c_str(), "treat_stdin()");
		}
		res = -1;
		return res;
	}

	ofd = open(path_ofn.c_str(),O_RDWR | O_CREAT | O_LARGEFILE, S_IRUSR | S_IWUSR);

	if( ofd == -1)
	{
		if(debug == 1)
		{
			sprintf(log_buf, "%s: cannot open file/device: %s, in: %s", progname.c_str(), path_ofn.c_str(), "treat_stdin()");
			log_str = log_buf;
			log(log_str);
		}
		else
		{
			fprintf (stderr, "%s: cannot open file/device: %s, in: %s\n", progname.c_str(), path_ofn.c_str(), "treat_stdin()");
		}
		rc = -1;
		return rc;
	}

	unzsplit_res = do_unzsplit(ifd, ofd, path_ifn, path_ofn);

	if(unzsplit_res == 0)
	{
		close_res = close(ifd);
		ifd = 0;
		res = 0;
		if(close_res != 0)
		{
			if(debug == 1)
			{
				sprintf(log_buf, "%s: error in closing the input file: %s, in: %s", progname.c_str(), path_ifn.c_str(), "close() after do_unzsplit()");
				log_str = log_buf;
				log(log_str);
			}
			else
			{
				fprintf (stderr, "%s: error in closing the input file: %s, in: %s\n", progname.c_str(), path_ifn.c_str(), "close() after do_unzsplit()");
			}
			close(ofd);
			ofd = 0;
			res = -1;
		}
	}

	else if(unzsplit_res == -3)
	{
		if(debug == 1)
		{
			sprintf(log_buf, "%s: error in accessing the file in do_stat(), in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
			log_str = log_buf;
			log(log_str);
		}
		else
		{
			fprintf (stderr, "%s: error in accessing the file in do_stat(), in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
		}
		close_in_out_files(ifd, ofd);
		res = -1;
	}
	else if(unzsplit_res == -5)
	{
		if(debug == 1)
		{
			sprintf(log_buf, "%s: error in writing to output file, in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
			log_str = log_buf;
			log(log_str);
		}
		else
		{
			fprintf (stderr, "%s: error in writing to output file, in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
		}
		close_in_out_files(ifd, ofd);
		res = -1;
	}
	else if(unzsplit_res == -6)
	{
		if(debug == 1)
		{
			sprintf(log_buf, "%s: error in initialization of inflation, in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
			log_str = log_buf;
			log(log_str);
		}
		else
		{
			fprintf (stderr, "%s: error in initialization of inflation, in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
		}
		close_in_out_files(ifd, ofd);
		res = -1;
	}
	else if(unzsplit_res == -7)
	{
		if(debug == 1)
		{
			sprintf(log_buf, "%s: unknown error, in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
			log_str = log_buf;
			log(log_str);
		}
		else
		{
			fprintf (stderr, "%s: unknown error, in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
		}
		close_in_out_files(ifd, ofd);
		res = -1;
	}
	else if(unzsplit_res == -8)
	{
		if(debug == 1)
		{
			sprintf(log_buf, "%s: stream structure was inconsistent, in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
			log_str = log_buf;
			log(log_str);
		}
		else
		{
			fprintf (stderr, "%s: stream structure was inconsistent, in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
		}
		close_in_out_files(ifd, ofd);
		res = -1;
	}
	else if(unzsplit_res == -11)
	{
		if(debug == 1)
		{
			sprintf(log_buf, "%s: there was not enough memory, in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
			log_str = log_buf;
			log(log_str);
		}
		else
		{
			fprintf (stderr, "%s: there was not enough memory, in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
		}
		close_in_out_files(ifd, ofd);
		res = -1;
	}
	else if(unzsplit_res == -12)
	{
		if(debug == 1)
		{
			sprintf(log_buf, "%s: corrupted input data, in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
			log_str = log_buf;
			log(log_str);
		}
		else
		{
			fprintf (stderr, "%s: corrupted input data, in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
		}
		close_in_out_files(ifd, ofd);
		res = -1;
	}

	else if(unzsplit_res == -13)
	{
		if(debug == 1)
		{
			sprintf(log_buf, "%s: I/O error occured, please try again with \"--noerror\" option; return=%d, in: %s", progname.c_str(), unzsplit_res, "do_unzsplit()");
			log_str = log_buf;
			log(log_str);
		}
		else
		{
			fprintf (stderr, "%s: I/O error occured, please try again with \"--noerror\" option; return=%d, in: %s\n", progname.c_str(), unzsplit_res, "do_unzsplit()");
		}
		close_in_out_files(ifd, ofd);
		res = -1;
	}
	else
	{
		if(debug == 1)
		{
			sprintf(log_buf, "%s: unexpected error..., in: %s return=%d", progname.c_str(), "do_unzsplit()", unzsplit_res);
			log_str = log_buf;
			log(log_str);
		}
		else
		{
			fprintf (stderr, "%s: unexpected error..., in: %s return=%d\n", progname.c_str(), "do_unzsplit()", unzsplit_res);
		}
		close_in_out_files(ifd, ofd);
		res = -1;
	}
  }

  return res;
}
